#!/usr/bin/env python
#
# Copyright (c) 2015 Serguei Glavatski ( verser  from cnc-club.ru )
# Copyright (c) 2020 Probe Screen NG Developers
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; If not, see <http://www.gnu.org/licenses/>.

import hal
import hal_glib

from .base import ProbeScreenBase


class ProbeScreenLengthMeasurement(ProbeScreenBase):
    # --------------------------
    #
    #  INIT
    #
    # --------------------------
    def __init__(self, halcomp, builder, useropts):
        super(ProbeScreenLengthMeasurement, self).__init__(halcomp, builder, useropts)

        self.lx_out = self.builder.get_object("lx_out")
        self.lx_in = self.builder.get_object("lx_in")
        self.ly_out = self.builder.get_object("ly_out")
        self.ly_in = self.builder.get_object("ly_in")
        self.spbtn_x_hint = self.builder.get_object("spbtn_x_hint")
        self.spbtn_y_hint = self.builder.get_object("spbtn_y_hint")
        
        self.spbtn_x_hint.set_value(
            self.prefs.getpref("x_hint", 1.0, float)
        )
        self.halcomp.newpin("x_hint", hal.HAL_FLOAT, hal.HAL_OUT)
        self.halcomp["x_hint"] = self.spbtn_x_hint.get_value()

        self.spbtn_y_hint.set_value(
            self.prefs.getpref("y_hint", 1.0, float)
        )
        self.halcomp.newpin("y_hint", hal.HAL_FLOAT, hal.HAL_OUT)
        self.halcomp["y_hint"] = self.spbtn_y_hint.get_value()

    def on_spbtn_x_hint_key_press_event(self, gtkspinbutton, data=None):
        self.on_common_spbtn_key_press_event("x_hint", gtkspinbutton, data)

    def on_spbtn_x_hint_value_changed(self, gtkspinbutton, data=None):
        self.on_common_spbtn_value_changed("x_hint", gtkspinbutton, data)

    def on_spbtn_y_hint_key_press_event(self, gtkspinbutton, data=None):
        self.on_common_spbtn_key_press_event("y_hint", gtkspinbutton, data)

    def on_spbtn_y_hint_value_changed(self, gtkspinbutton, data=None):
        self.on_common_spbtn_value_changed("y_hint", gtkspinbutton, data)

    # --------------
    # Length Buttons
    # --------------

    # Lx OUT
    @ProbeScreenBase.ensure_errors_dismissed
    def on_lx_out_released(self, gtkbutton, data=None):
    
        estimated_width = self.spbtn_x_hint.get_value()
        a = self.current_position_with_offsets()
        # move X - edge_length- xy_clearance
        tmpx = estimated_width / 2 + self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(-tmpx, 0)
        xpres = self.probe_x(direction = 1, move_enable = False, s = s)
        s = self.move_x(a[0])
        s += self.move_rel_xy(tmpx, 0)
        xmres = self.probe_x(direction = -1, move_enable = False, orient_enable = False, s = s)
        xcres = 0.5 * (xpres + xmres)
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmXcXpLx",
            xm=xmres,
            xc=xcres,
            xp=xpres,
            lx=self.length_x(xm=xmres, xp=xpres),
        )
        # go to the new center of X
        self.gcode(self.move_x(xcres))
        self.set_zerro("X")


    # Ly OUT
    @ProbeScreenBase.ensure_errors_dismissed
    def on_ly_out_released(self, gtkbutton, data=None):
        estimated_width = self.spbtn_y_hint.get_value()
        a = self.current_position_with_offsets()
        # move Y - edge_length- xy_clearance
        tmpy = estimated_width / 2 + self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(0, -tmpy)
        ypres = self.probe_y(direction = 1, move_enable = False, s = s)
        s = self.move_y(a[1])
        s += self.move_rel_xy(0, tmpy)
        ymres = self.probe_y(direction = -1, move_enable = False, orient_enable = False, s = s)
        ycres = 0.5 * (ypres + ymres)
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "YmYcYpLy",
            ym=ymres,
            yc=ycres,
            yp=ypres,
            ly=self.length_y(ym=ymres, yp=ypres),
        )
        # go to the new center of Y
        self.gcode(self.move_y(ycres))
        self.set_zerro("Y")

    # Lx IN
    @ProbeScreenBase.ensure_errors_dismissed
    def on_lx_in_released(self, gtkbutton, data=None):
        estimated_width = self.spbtn_x_hint.get_value()
        a = self.current_position_with_offsets()
        # move X - edge_length+ xy_clearance
        tmpx = estimated_width / 2 - self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(-tmpx, 0)
        xpres = self.probe_x(direction = -1, move_enable = False, s = s)
        s = self.move_x(a[0]) # return to the start
        s += self.move_rel_xy(tmpx, 0)
        xmres = self.probe_x(direction = 1, move_enable = False, orient_enable = False, s = s)
        xcres = 0.5 * (xpres + xmres)
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmXcXpLx",
            xm=xmres,
            xc=xcres,
            xp=xpres,
            lx=self.length_x(xm=xmres, xp=xpres),
        )
        # go to the new center of X
        self.gcode(self.move_x(xcres))
        self.set_zerro("X")

    # Ly IN
    @ProbeScreenBase.ensure_errors_dismissed
    def on_ly_in_released(self, gtkbutton, data=None):
        estimated_width = self.spbtn_y_hint.get_value()
        a = self.current_position_with_offsets()
        # move Y - edge_length+ xy_clearance
        tmpy = estimated_width / 2 - self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(0, -tmpy)
        ymres = self.probe_y(direction = -1, move_enable = False, s = s)
        s = self.move_y(a[1])
        s += self.move_rel_xy(0, tmpy)
        ypres = self.probe_y(direction = 1, move_enable = False, orient_enable = False, s = s)
        ycres = 0.5 * (ypres + ymres)
        
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "YmYcYpLy",
            ym=ymres,
            yc=ycres,
            yp=ypres,
            ly=self.length_y(ym=ymres, yp=ypres),
        )
        # go to the new center of Y
        self.gcode(self.move_y(ycres))
        self.set_zerro("Y")
