#!/usr/bin/env python
#
# Copyright (c) 2015 Serguei Glavatski ( verser  from cnc-club.ru )
# Copyright (c) 2020 Probe Screen NG Developers
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; If not, see <http://www.gnu.org/licenses/>.

from .base import ProbeScreenBase


class ProbeScreenWorkpieceMeasurement(ProbeScreenBase):

    # --------------------------
    #
    #  INIT
    #
    # --------------------------
    def __init__(self, halcomp, builder, useropts):
        super(ProbeScreenWorkpieceMeasurement, self).__init__(
            halcomp, builder, useropts
        )

        self.xpym = self.builder.get_object("xpym")
        self.ym = self.builder.get_object("ym")
        self.xmym = self.builder.get_object("xmym")
        self.xp = self.builder.get_object("xp")
        self.center = self.builder.get_object("center")
        self.xm = self.builder.get_object("xm")
        self.xpyp = self.builder.get_object("xpyp")
        self.yp = self.builder.get_object("yp")
        self.xmyp = self.builder.get_object("xmyp")
        self.hole = self.builder.get_object("hole")

    # --------------  Command buttons -----------------
    #               Measurement outside
    # -------------------------------------------------
    # X+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xp_released(self, gtkbutton, data=None):
        xres = self.probe_x(direction = 1)
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XpLx",
            xp=xres,
            lx=self.length_x(xp=xres),
        )
        self.set_zerro("X")
            
    # Y+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_yp_released(self, gtkbutton, data=None):
        yres = self.probe_y(direction = 1)
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "YpLy",
            yp=yres,
            ly=self.length_y(yp=yres),
        )
        self.set_zerro("Y")
            

    # X-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xm_released(self, gtkbutton, data=None):
        xres = self.probe_x(direction = -1)
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmLx",
            xm=xres,
            lx=self.length_x(xm=xres),
        )
        self.set_zerro("X")
            
    # Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_ym_released(self, gtkbutton, data=None):
        yres = self.probe_y(direction = -1)
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "YmLy",
            ym=yres,
            ly=self.length_y(ym=yres),
        )
        self.set_zerro("Y")
            
    # Corners
    # Move Probe manual under corner 2-3 mm
    # X+Y+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xpyp_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(-self.halcomp["ps_xy_clearance"], self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = 1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] + self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(tmpxy, -tmpxy)
        yres = self.probe_y(direction = 1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XpLxYpLy",
            xp=xres,
            lx=self.length_x(xp=xres),
            yp=yres,
            ly=self.length_y(yp=yres),
        )
        self.set_zerro("XY")
            
    # X+Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xpym_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(-self.halcomp["ps_xy_clearance"], -self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = 1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] + self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(tmpxy, tmpxy)
        yres = self.probe_y(direction = -1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XpLxYmLy",
            xp=xres,
            lx=self.length_x(xp=xres),
            ym=yres,
            ly=self.length_y(ym=yres),
        )
        self.set_zerro("XY")

    # X-Y+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xmyp_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(self.halcomp["ps_xy_clearance"], self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = -1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] + self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(-tmpxy, -tmpxy)
        yres = self.probe_y(direction = 1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmLxYpLy",
            xm=xres,
            lx=self.length_x(xm=xres),
            yp=yres,
            ly=self.length_y(yp=yres),
        )           
        self.set_zerro("XY")

    # X-Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xmym_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(self.halcomp["ps_xy_clearance"], -self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = -1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] + self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(-tmpxy, tmpxy)
        yres = self.probe_y(direction = -1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmLxYmLy",
            xm=xres,
            lx=self.length_x(xm=xres),
            ym=yres,
            ly=self.length_y(ym=yres),
        )           
        self.set_zerro("XY")

    # Center X+ X- Y+ Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xy_center_released(self, gtkbutton, data=None):

        lengths = self.probe_circle(direction = 1,
            dia_estimate = 2,
            num_points = 5,
            raw_output = True
            )
        
        self.add_history_text(str(lengths))
        # xp, xn, yp, yn = self.probe_xy(
            # direction = 1,
            # x_estimate = self.halcomp["ps_edge_length"] * 2,
            # y_estimate = self.halcomp["ps_edge_length"] * 2,
            # move_center = True
        # )
        # self.add_history(
            # gtkbutton.get_tooltip_text(),
            # "XmXcXpLxYmYcYpLyD",
            # xm=xn,
            # xc=(xp + xn)/2,
            # xp=xp,
            # lx=self.length_x(xm=xn, xp=xp),
            # ym=yn,
            # yc=(yp + yn)/2,
            # yp=yp,
            # ly=self.length_y(ym=yn, yp=yp),
            # d=yp - yn
        # )
        # self.set_zerro("XY")

    # --------------  Command buttons -----------------
    #               Measurement inside
    # -------------------------------------------------

    # Corners
    # Move Probe manual under corner 2-3 mm
    # X+Y+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xpyp1_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(-self.halcomp["ps_xy_clearance"], -self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = 1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] - self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(-tmpxy, tmpxy)
        yres = self.probe_y(direction = 1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XpLxYpLy",
            xp=xres,
            lx=self.length_x(xp=xres),
            yp=yres,
            ly=self.length_y(yp=yres),
        )           
        self.set_zerro("XY")

    # X+Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xpym1_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(-self.halcomp["ps_xy_clearance"], self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = 1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] - self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(-tmpxy, -tmpxy)
        yres = self.probe_y(direction = -1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XpLxYmLy",
            xp=xres,
            lx=self.length_x(xp=xres),
            ym=yres,
            ly=self.length_y(ym=yres),
        )
        self.set_zerro("XY")

    # X-Y+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xmyp1_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(self.halcomp["ps_xy_clearance"], -self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = -1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] - self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(tmpxy, tmpxy)
        yres = self.probe_y(direction = 1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmLxYpLy",
            xm=xres,
            lx=self.length_x(xm=xres),
            yp=yres,
            ly=self.length_y(yp=yres),
        )           
        self.set_zerro("XY")

    # X-Y-
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xmym1_released(self, gtkbutton, data=None):
        s = self.move_rel_xy(self.halcomp["ps_xy_clearance"], self.halcomp["ps_edge_length"])
        xres = self.probe_x(direction = -1, move_enable = False, s = s)
        tmpxy = self.halcomp["ps_edge_length"] - self.halcomp["ps_xy_clearance"]
        s = self.move_rel_xy(tmpxy, -tmpxy)
        yres = self.probe_y(direction = -1, move_enable = False, orient_enable = False, s = s)
        self.gcode(self.move_xy(xres, yres))
        self.add_history(
            gtkbutton.get_tooltip_text(),
            "XmLxYmLy",
            xm=xres,
            lx=self.length_x(xm=xres),
            ym=yres,
            ly=self.length_y(ym=yres),
        )
        self.set_zerro("XY")
        
    # Hole Xin- Xin+ Yin- Yin+
    @ProbeScreenBase.ensure_errors_dismissed
    def on_xy_hole_released(self, gtkbutton, data=None):
        lengths = self.probe_circle(direction = -1,
            dia_estimate = 2,
            num_points = 5,
            raw_output = True
            )
        
        self.add_history_text(str(lengths))
        # xp, xn, yp, yn = self.probe_xy(
            # direction = -1,
            # x_estimate = self.halcomp["ps_edge_length"] * 2,
            # y_estimate = self.halcomp["ps_edge_length"] * 2,
            # move_center = True
        # )
        # self.add_history(
            # gtkbutton.get_tooltip_text(),
            # "XmXcXpLxYmYcYpLyD",
            # xm=xn,
            # xc=(xp + xn)/2,
            # xp=xp,
            # lx=self.length_x(xm=xn, xp=xp),
            # ym=yn,
            # yc=(yp + yn)/2,
            # yp=yp,
            # ly=self.length_y(ym=yn, yp=yp),
            # d=yp - yn
        # )
        # self.set_zerro("XY")
